@@ -5,10 +5,16 @@ import threading
 import leselys
 import copy
 
+import requests
+from urlparse import urlparse
+from urlparse import urljoin
+
 from leselys.helpers import u
 from leselys.helpers import get_datetime
 from leselys.helpers import get_dicttime
+from leselys.feed_finder import FeedFinder
 
+feedparser.USER_AGENT = "Leselys/%s +https://github.com/socketubs/leselys" % leselys.__version__
 storage = leselys.core.storage
 
 #########################################################################
@@ -85,24 +91,32 @@ class Refresher(threading.Thread):
         self.data = feedparser.parse(self.feed['url'])
 
         local_update = get_datetime(self.feed['last_update'])
+        remote_update = False
         if self.data.feed.get('updated_parsed'):
             remote_update = get_datetime(self.data.feed.updated_parsed)
             remote_update_raw = get_dicttime(self.data.feed.updated_parsed)
-        elif self.data.get('updated_parsed'):
-            remote_update = get_datetime(self.data.updated_parsed)
-            remote_update_raw = get_dicttime(self.data.updated_parsed)
-        elif self.data.feed.get('published_parsed'):
-            remote_update = get_datetime(self.data.feed.published_parsed)
-            remote_update_raw = get_dicttime(self.data.feed.published_parsed)
-        elif self.data.get('published_parsed'):
-            remote_update = get_datetime(self.data.published_parsed)
-            remote_update_raw = get_dicttime(self.data.published_parsed)
-        else:
+        if self.data.get('updated_parsed'):
+            if remote_update:
+                if get_datetime(self.data.updated_parsed) > remote_update:
+                    remote_update = get_datetime(self.data.updated_parsed)
+                    remote_update_raw = get_dicttime(self.data.updated_parsed)
+        if self.data.feed.get('published_parsed'):
+            if remote_update:
+                if get_datetime(self.data.feed.published_parsed) > remote_update:
+                    remote_update = get_datetime(self.data.feed.published_parsed)
+                    remote_update_raw = get_dicttime(self.data.feed.published_parsed)
+        if self.data.get('published_parsed'):
+            if remote_update:
+                if get_datetime(self.data.published_parsed > remote_update):
+                    remote_update = get_datetime(self.data.published_parsed)
+                    remote_update_raw = get_dicttime(self.data.published_parsed)
+
+        if not remote_update:
             remote_update = datetime.datetime.now()
             remote_update_raw = get_dicttime(remote_update.timetuple())
 
         if remote_update > local_update:
-            print(':: %s is outdated' % self.feed['title'])
+            print(':: %s is outdated' % self.feed['title'].encode('utf-8'))
             readed = []
             for entry in storage.get_stories(self.feed['_id']):
                 if entry['read']:
@@ -128,20 +142,40 @@ class Refresher(threading.Thread):
 
 
 class Reader(object):
-    """The Reader object is the subscriptions manager, it handles all
+    """The Reader object is the feeds manager, it handles all
     new feed, read/unread state and refresh feeds
     """
 
+    def get_feed(self, url):
+        """Given url might be point to http document or to actual feed. In case
+        of http document, we try to find first feed auto discovery url.
+        """
+        stripped = url.strip()
+        resp = requests.get(stripped)
+        feed = feedparser.parse(resp.text)
+        if feed.version != '':
+            return feed
+
+        urls = FeedFinder.parse(resp.text)
+        feed_url = ''
+        if len(urls) > 0:
+            # Each url is tuple where href is first element.
+            # NOTE : Sites might have several feeds available and we are just
+            # naively picking first one found.
+            feed_url = urls[0][0]
+            if urlparse(feed_url)[1] == '':
+                # We have empty 'netloc', meaning we have relative url
+                feed_url = urljoin(stripped, feed_url)
+        return feedparser.parse(feed_url)
+
     def add(self, url):
-        url = url.strip()
-        feed = feedparser.parse(url)
+        feed = self.get_feed(url)
 
         # Bad feed
-        if not feed.feed.get('title', False):
+        if feed.version == '' or not feed.feed.get('title'):
             return {'success': False, 'output': 'Bad feed'}
 
         title = feed.feed['title']
-
         feed_id = storage.get_feed_by_title(title)
         if not feed_id:
             if feed.feed.get('updated_parsed'):
@@ -167,6 +201,7 @@ class Reader(object):
         return {
             'success': True,
             'title': title,
+            'url': url,
             'feed_id': feed_id,
             'output': 'Feed added',
             'counter': len(feed['entries'])}
@@ -203,24 +238,45 @@ class Reader(object):
                       reverse=True)
         return unreaded + readed
 
-    def get_subscriptions(self):
+    def get_feeds(self):
         feeds = []
         for feed in storage.get_feeds():
             feeds.append({'title': feed['title'],
                           'id': feed['_id'],
+                          'url': feed['url'],
                           'counter': self.get_unread(feed['_id'])
                           })
         return feeds
 
-    def refresh_all(self):
-        for subscription in storage.get_feeds():
-            refresher = Refresher(subscription)
-            refresher.start()
-        return []
+    def refresh(self, feed_id):
+        feed = storage.get_feed_by_id(feed_id)
+        refresher = Refresher(feed)
+        refresher.start()
+        refresher.join()
+        feed['counter'] = self.get_unread(feed_id)
+        return {'success': True, 'content': feed}
 
     def get_unread(self, feed_id):
         return len(storage.get_feed_unread(feed_id))
 
+    def mark_all_read(self, feed_id):
+        stories = storage.get_stories(feed_id)
+        if not stories:
+            return {'success': False, "content": "Feed not found"}
+        for story in storage.get_stories(feed_id):
+            story['read'] = True
+            storage.update_story(story['_id'], copy.copy(story))
+        return {'success': True, "content": "All feed stories updated"}
+
+    def mark_all_unread(self, feed_id):
+        stories = storage.get_stories(feed_id)
+        if not stories:
+            return {'success': False, "content": "Feed not found"}
+        for story in storage.get_stories(feed_id):
+            story['read'] = False
+            storage.update_story(story['_id'], copy.copy(story))
+        return {'success': True, "content": "All feed stories updated"}
+
     def read(self, story_id):
         """
         Return story content, set it at readed state and give
